#ifndef __TWOOSTWO_PAGE_MANAGER__
#define __TWOOSTWO_PAGE_MANAGER__

#define MAX_AREAS 256

#define DEFAULT_DESCRIPTOR  0x00000DEF

#define AREA_FREE 0x00000000
#define INVALID_AREA 0x80000000
#define AREA_ALLOCATED 0x00000001
#define AREA_LOCKED 0x00000002

#define PAGE_UNLOCK_FAILED -5
#define PAGE_ALLOC_FAILED -4
#define MEM_ALLOC_FAILED -3
#define NO_MORE_AREAS  -2
#define GENERIC_ERROR   -1
#define SUCCESS   0

#define ALLOCATION_FLAGS  (GFP_HIGHUSER)


typedef struct __page_repository__
{
 int uiPagesCnt;
 struct page** pPages;
 void* pVirtual;
 unsigned int UserAreaDesc;
 unsigned int uiFlags;
} page_repository_t;

typedef struct __areas_data__
{
 unsigned int uiAreasCnt;
 page_repository_t Pages[MAX_AREAS];
} areas_data_t;

// Initialize structures
int InitPageManager(areas_data_t* Areas);
// Delete all mappings, free all pages and deinitialize structures
int EndPageManager(areas_data_t* Areas);
// Allocate area ( return area_no, descriptor used for debug and/or area managing )
int AllocateArea(areas_data_t* Areas, unsigned int size, unsigned int AreaDescriptor);
// Free area ( also unmap if mapped )
int FreeArea(areas_data_t* Areas, unsigned int area);
// Get first free area
int GetFreeArea(areas_data_t* Areas);
// Get pysical address of page no 
unsigned int GetPhyPage(areas_data_t* Areas, unsigned int area, unsigned int page);
// Lock selected area
unsigned int LockArea(areas_data_t* Areas, unsigned int area);
// Unlock selected area
unsigned int UnlockArea(areas_data_t* Areas, unsigned int area);

#endif
