#include <sys/time.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <vmTest.h>

#ifndef PAGE_SIZE
 #define PAGE_SIZE 4096
#endif

// 100M
#define ALLOC_BLOCK ((PAGE_SIZE) * 25600)

#define MAX_BLOCKS 2
#define FILLER_DATA (0xBE)

static int iDev = 0;
static ioctl_request_t request;

void DisplayError(int errorn)
{
        switch(errorn)
 {
 case PAGE_UNLOCK_FAILED:
  printf("Error: PAGE_UNLOCK_FAILED\n");
  break; 
 case PAGE_ALLOC_FAILED: 
  printf("Error: PAGE_ALLOC_FAILED\n");
  break; 
 case MEM_ALLOC_FAILED: 
  printf("Error: MEM_ALLOC_FAILED\n");
  break; 
 case NO_MORE_AREAS:  
  printf("Error: NO_MORE_AREAS\n");
  break; 
 case GENERIC_ERROR:
  printf("Error: GENERIC_ERROR\n");
  break; 
 case SUCCESS:
  printf("Error: SUCCESS\n");
  break; 
 default:
  printf("Unrecodnized error 0x%x\n",errno);
 }
}

int main(int argc, char* argv[])
{
 int area[1024], iIndex = 0, iiIndex, iJindex;
 char pBuffer[1024];
 void* mmaped[1024];

 printf("Page-man library tester 0.0\n");
 printf("Trying to open device... ");
 
 iDev = open("/dev/vmTest", O_RDWR);

 if (iDev < 0)
 {
  printf("failed!\n");
  exit(-1);
 } else
  printf("ok.\n");

 for (iIndex = 0; iIndex < MAX_BLOCKS; iIndex++)
 {
  printf("Allocating page pool no. %u... ", iIndex);
  request.uCtlCode = IOCTL_ALLOC_AREA;
  request.pSysBuffer = (void*)pBuffer;
  *(int*)pBuffer = ALLOC_BLOCK;
         if (ioctl(iDev, IOCTL_ALLOC_AREA, &request)<0)
         {
          printf("failed.");
          DisplayError(*(int*)pBuffer);
          break;
  } else       
  {
   area[iIndex] = *(int*)pBuffer;
   printf("ok.\n");
  }
 }
   
 if (iIndex == MAX_BLOCKS)
 {
  // allocated good
  for(iiIndex = 0; iiIndex < MAX_BLOCKS; iiIndex++)
  {
   printf("Mapping page pool no. %u... ", iiIndex);
   request.uCtlCode = IOCTL_MAP_AREA;
   request.pSysBuffer = (void*)pBuffer;
   *(int*)pBuffer = iiIndex;
          if (ioctl(iDev, IOCTL_MAP_AREA, &request)<0)
          {
           printf("ioctl failed.");
           DisplayError(*(int*)pBuffer);
           break;
   } else       
   {
                         mmaped[iiIndex] = mmap(NULL, ALLOC_BLOCK,
                                               PROT_READ | PROT_WRITE, 
                                               MAP_SHARED, iDev, 0);
                                if (mmaped[iiIndex]==MAP_FAILED)
                                 printf("failed.\n");
                                else
     printf("ok.\n");
   }
  }
  

  if (iiIndex == MAX_BLOCKS)
  {
                 printf("Test areas.\n");
   
   for(iJindex = 0; iJindex < MAX_BLOCKS; iJindex++)
   {
           unsigned char* pBuf = (unsigned char*)mmaped[iJindex];
    printf("Write mmaped area %u... ", iJindex);
    memset(mmaped[iJindex], FILLER_DATA, ALLOC_BLOCK);
    printf("ok.\n");
    {
     printf("Check data written in area %u... ", iJindex);
            int iBlockIndex;
     for(iBlockIndex = 0; iBlockIndex < ALLOC_BLOCK; iBlockIndex++)
     {
      if (pBuf[iBlockIndex] != FILLER_DATA)
      {
       printf("failed.\n\tTest failed at byte %u", iBlockIndex);
       break;
      }
     }

     if (iBlockIndex == ALLOC_BLOCK)
      printf("ok.\n");
    }
   }
                 printf("Test areas complete.\n");
                 iiIndex--;
  }
         printf("Unmap areas.\n");

#if 1
  for(iJindex = 0; iJindex < (iiIndex+1); iJindex++)
  {
   printf("Unmapping page pool no. %u... ", iJindex);
                 munmap(mmaped[iJindex], ALLOC_BLOCK);
   printf("ok.\n");
  }
#endif
  iIndex--;
 }
 
 for (iiIndex = iIndex; iiIndex >= 0 ; iiIndex--)
 {
  printf("Freeing page pool no. %u... ", iiIndex);
  request.uCtlCode = IOCTL_FREE_AREA;
  request.pSysBuffer = (void*)pBuffer;
  *(int*)pBuffer = area[iiIndex];
         if (ioctl(iDev, IOCTL_FREE_AREA, &request)<0)
         {
          printf("failed.");
          DisplayError(*(int*)pBuffer);
  } else       
   printf("ok.\n");
 }

 printf("Trying to close device... ");
 close(iDev);
 printf("ok.\n");

 printf("Waiting for a keypress...\n");
 scanf("%u", *(int*)pBuffer);
};
